package edu.travel.tenant.web;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import edu.travel.remote.feign.mode.dto.tenant.AddOrUpdateTenantDto;
import edu.travel.remote.feign.mode.dto.tenant.AssignRolesOrMenusDto;
import edu.travel.remote.feign.mode.dto.tenant.EduTenantPageDto;
import edu.travel.remote.feign.mode.vo.tenant.AdminInfoRoleMenuVo;
import edu.travel.remote.feign.mode.vo.tenant.EduTenantVo;
import edu.travel.remote.feign.mode.vo.tenant.MenuTreeVo;
import edu.travel.remote.feign.mode.vo.tenant.SysRoleVo;
import edu.travel.resp.BaseResponse;
import edu.travel.resp.PageResponse;
import edu.travel.tenant.TenantRemoteController;
import edu.travel.tenant.entity.EduTenant;
import edu.travel.tenant.entity.SysRole;
import edu.travel.tenant.entity.SysUserRole;
import edu.travel.tenant.service.ISysUserRoleService;
import edu.travel.tenant.service.ITenantService;
import edu.travel.web.BaseController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/user")
public class TenantController  extends BaseController<EduTenant>  {
    @Autowired
    private ITenantService tenantService;
    @Autowired
    private ISysUserRoleService sysUserRoleService;

    @GetMapping("/getTenantByPhoneNumber")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<EduTenantVo> getTenantByPhoneNumber() {
        EduTenant principal = (EduTenant) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
        EduTenant tenant = tenantService.getTenantByPhoneNumber(principal.getTenantPhone());
        EduTenantVo vo = BeanUtil.copyProperties(tenant, EduTenantVo.class);
        return PageResponse.out(200, "success", vo);
    }

    /**
     * 新增用户
     *
     * @param addOrUpdateTenantDto
     * @return
     */
    @PostMapping("/addTenant")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<EduTenantVo> addTenant(@RequestBody AddOrUpdateTenantDto addOrUpdateTenantDto) {
        EduTenant eduTenant = tenantService.addTenant(addOrUpdateTenantDto);
        return PageResponse.out(200, "success", BeanUtil.copyProperties(eduTenant, EduTenantVo.class));
    }

    /**
     * 通过ids删除用户
     *
     * @param ids
     * @return
     */
    @PostMapping("/deleteTenantByIds")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<Boolean> deleteTenant(@RequestBody List<Long> ids) {
//        List<Long> list = ids.get("ids");
        HashSet<Long> set = new HashSet<>(ids);
        tenantService.deleteByIds(set);
        return PageResponse.out(200, "success", true);
    }

    /**
     * 通过id删除用户
     *
     * @param id
     * @return
     */
    @PostMapping("/deleteTenantById")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<Boolean> deleteTenant(Long id) {
        Set<Long> list = new HashSet<>();
        list.add(id);
        tenantService.deleteByIds(list);
        return PageResponse.out(200, "success", true);
    }

    /**
     * 修改用户信息
     *
     * @param addOrUpdateTenantDto
     * @return
     */
    @PostMapping("/updateTenantById")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<Boolean> updateTenant(@RequestBody AddOrUpdateTenantDto addOrUpdateTenantDto) {
        tenantService.updateTenant(addOrUpdateTenantDto);
        return PageResponse.out(200, "success", true);
    }

    /**
     * 通过id查询商城用户信息
     *
     * @param id
     * @return
     */
    @GetMapping("/getTenantById")
////    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<EduTenantVo> getTenantById(Long id) {
        EduTenant tenant = tenantService.getTenantById(id);
        EduTenantVo eduTenantVo = BeanUtil.copyProperties(tenant, EduTenantVo.class);
        return PageResponse.out(200, "success", eduTenantVo);
    }

    /**
     * 查询登录用户信息
     *
     * @return
     */
    @GetMapping("/getLoginTenantInfo")
////    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<EduTenantVo> getLoginTenantInfo() {
        EduTenant principal = (EduTenant) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
        EduTenant tenant = tenantService.getTenantById(principal.getId());
        EduTenantVo eduTenantVo = BeanUtil.copyProperties(tenant, EduTenantVo.class);
        return PageResponse.out(200, "success", eduTenantVo);
    }


    /**
     * 获取用户分页列表
     *
     * @param tenantDto
     * @return
     */
    @GetMapping("/getTenantPageList")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<List<EduTenantVo>> getTenantList( EduTenantPageDto tenantDto) {

        IPage<EduTenant> page = new Page<>(tenantDto.getCurrentPage(), tenantDto.getPageSize());

        LambdaQueryWrapper<EduTenant> queryWrapper = new LambdaQueryWrapper<EduTenant>()
                .eq(EduTenant::getDeleteFlag, 0)
                .eq(tenantDto.getId() != null, EduTenant::getId, tenantDto.getId())
                .like(tenantDto.getTenantName() != null && !tenantDto.getTenantName().isEmpty(), EduTenant::getTenantName, tenantDto.getTenantName());
        IPage<EduTenant> tenantIPage = tenantService.dictMapPage(page, queryWrapper);
        List<EduTenantVo> eduTenantVos = BeanUtil.copyToList(tenantIPage.getRecords(), EduTenantVo.class);
        List<String> userIds = eduTenantVos.stream().map(EduTenantVo::getId).collect(Collectors.toList());
        List<SysUserRole> roleList = sysUserRoleService.list(new LambdaQueryWrapper<SysUserRole>()
                .in(SysUserRole::getUserId, userIds));
        Map<Long, List<Long>> userRolesMap = roleList.stream()
                .collect(Collectors.groupingBy(
                        SysUserRole::getUserId,
                        Collectors.mapping(SysUserRole::getRoleId, Collectors.toList())
                ));
        eduTenantVos.forEach(eduTenantVo -> {
            List<Long> roleIds = userRolesMap.get(eduTenantVo.getId());
            if (roleIds != null) {
                eduTenantVo.setRoleIdList(roleIds);
            }
        });

        return PageResponse.out(200, "success", eduTenantVos, (int) tenantIPage.getTotal(), (int) tenantIPage.getSize());
    }

    /**
     * 根据用户id获取用户角色列表(未分页)
     *
     * @return
     */
    @GetMapping("/getRoleListByUserId")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<List<SysRoleVo>> getRoleListByUserId() {
        EduTenant principal = (EduTenant) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
        List<SysRole> roleList = tenantService.getRoleListByUserId(principal);
        List<SysRoleVo> sysRoleVos = BeanUtil.copyToList(roleList, SysRoleVo.class);
        return PageResponse.out(200, "success", sysRoleVos);
    }

    /**
     * 根据用户id给用户分配角色(可以多个角色)
     *
     * @param assignRolesOrMenusDto
     * @return
     */
    @PostMapping("/assignRolesToUser")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<Boolean> assignRolesToUser(@RequestBody AssignRolesOrMenusDto assignRolesOrMenusDto) {
        EduTenant principal = (EduTenant) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
        tenantService.assignRolesToUser(assignRolesOrMenusDto, principal);
        return PageResponse.out(200, "success", true);
    }

    /**
     * 根据用户id给用户修改角色(可以多个角色)
     *
     * @param assignRolesOrMenusDto
     * @return
     */
    @PostMapping("updateRolesToUser")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<Boolean> updateRolesToUser(@RequestBody AssignRolesOrMenusDto assignRolesOrMenusDto) {
        EduTenant principal = (EduTenant) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
        tenantService.updateRolesToUser(assignRolesOrMenusDto, principal);
        return PageResponse.out(200, "success", true);
    }

    /**
     * 获取管理用户的信息、角色列表、菜单树形
     *
     * @return
     */
    @GetMapping("/getTenantInfoRoleListMenuTree")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<AdminInfoRoleMenuVo> getTenantInfoRoleListMenuTree() {
        EduTenant principal = (EduTenant) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
        EduTenant tenant = tenantService.getTenantById(principal.getId());
        EduTenantVo eduTenantVo = BeanUtil.copyProperties(tenant, EduTenantVo.class);
        List<SysRole> roleList = tenantService.getRoleListByUserId(principal);
        List<SysRoleVo> roleVoList = BeanUtil.copyToList(roleList, SysRoleVo.class);
        List<MenuTreeVo> menuTreeVos = tenantService.getTenantPageTreeList(principal);
        AdminInfoRoleMenuVo vo = new AdminInfoRoleMenuVo();
        vo.setTenantVo(eduTenantVo);
        vo.setRoleVoList(roleVoList == null ? new ArrayList<>() : roleVoList);
        vo.setMenuTreeVo(menuTreeVos);
        return PageResponse.out(200, "success", vo);
    }

    /**
     * 单个禁用用户
     *
     * @param id
     * @return
     */
    @PostMapping("/disableTenantById")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<Boolean> disableTenantById(Long id) {
        Assert.isTrue(ObjectUtil.isNotEmpty(id), "请选择用户");
        Set<Long> list = new HashSet<>();
        list.add(id);
        tenantService.disableTenantByIds(list);
        return PageResponse.out(200, "success", true);
    }

    /**
     * 单个启用用户
     *
     * @param id
     * @return
     */
    @PostMapping("/enableTenantById")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<Boolean> enableTenantById(Long id) {
        Assert.isTrue(ObjectUtil.isNotEmpty(id), "请选择用户");
        Set<Long> list = new HashSet<>();
        list.add(id);
        tenantService.enableTenantByIds(list);
        return PageResponse.out(200, "success", true);
    }

    /**
     * 批量启用用户
     *
     * @param ids
     * @return
     */
    @PostMapping("/enableTenantByIds")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<Boolean> enableTenantByIds(@RequestBody List<Long> ids) {
        tenantService.enableTenantByIds(new HashSet<>(ids));
        return PageResponse.out(200, "success", true);
    }

    /**
     * 批量禁用用户
     *
     * @param ids
     * @return
     */
    @PostMapping("/disableTenantByIds")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<Boolean> disableTenantByIds(@RequestBody List<Long> ids) {
        tenantService.disableTenantByIds(new HashSet<>(ids));
        return PageResponse.out(200, "success", true);
    }
}

