package edu.travel.config;

import edu.travel.RSAUtill;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.crypto.password.NoOpPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.oauth2.provider.token.TokenStore;
import org.springframework.security.oauth2.provider.token.store.JwtAccessTokenConverter;
import org.springframework.security.oauth2.provider.token.store.JwtTokenStore;

import java.security.PrivateKey;

@Configuration
public class TokenConfig {
    @Value("${OAUTH_KEY}")
    private String key;
    @Value("${PRIVATE_KEY}")
    private String privateKey;
    @Bean
    public PasswordEncoder passwordEncoder(){
        return NoOpPasswordEncoder.getInstance();
    }
    @Bean
    public TokenStore tokenStore() {
        //JWT令牌存储方案
        return new JwtTokenStore(accessTokenConverter());
    }

    @Bean
    public JwtAccessTokenConverter accessTokenConverter() {
        try {
            PrivateKey privateKeyFromString = RSAUtill.getPrivateKeyFromString(privateKey);
            String decrypt = RSAUtill.decrypt(key, privateKeyFromString);
            JwtAccessTokenConverter converter = new JwtAccessTokenConverter();
            //对称秘钥，资源服务器使用该秘钥来验证
//        converter.setKeyPair(keyPair());
            converter.setSigningKey(decrypt);
            return converter;

        }catch (Exception e){
            e.printStackTrace();
            return null;
        }

    }
}